""" testen voor de package waterstand """
import json
import unittest
from unittest.mock import patch, MagicMock, ANY

from requests.exceptions import HTTPError, Timeout, RequestException

import waterstand


def create_mock_response(filenaam):
  """ maak een response van de testdata """
  with open(filenaam, 'r', encoding='utf-8') as fid:
    testdata = json.loads(fid.read().encode('utf-8'))

  print(testdata)

  mock_response = MagicMock()
  mock_response.getcode.return_value = 200
  mock_response.json.return_value = testdata
  mock_response.__enter__.return_value = mock_response
  return mock_response


class TestWaterstand(unittest.TestCase):
  """ Class voor het testen van de package waterstand """

  @patch('requests.get')
  def test_haalwaterstand(self, mock_requestsget):
    """ test van de normale flow """
    mock_requestsget.return_value = create_mock_response('tests/testdata.json')

    response = waterstand.haalwaterstand('Katerveer', 'KATV')
    verwacht = {'resultaat': 'OK', 'tijd': '23-11 16:50', 'nu': 84.0, 'morgen': 89.0}

    self.assertEqual(response, verwacht)

  @patch('requests.get')
  def test_haalwaterstand_timeformat(self, mock_requestsget):
    """ test 2 van de normale flow """
    mock_requestsget.return_value = create_mock_response('tests/testdata2.json')

    response = waterstand.haalwaterstand('Katerveer', 'KATV')
    verwacht = {'resultaat': 'OK', 'tijd': '23-11 16:50', 'nu': 84.0, 'morgen': 89.0}

    self.assertEqual(response, verwacht)

  @patch('requests.get', side_effect=HTTPError('', 0, '', None, None))
  def test_haalwaterstand_httperror(self, mock_requestsget):
    """ test van een HTTP-fout bij het ophalen """
    response = waterstand.haalwaterstand('Katerveer', 'KATV')

    expected = {'resultaat': 'NOK', 'error': ANY}
    self.assertEqual(response, expected)
    assert mock_requestsget.called

  @patch('requests.get', side_effect=TimeoutError)
  def test_haalwaterstand_timeouterror(self, mock_requestsget):
    """ test van een time-out bij het ophalen """
    response = waterstand.haalwaterstand('Katerveer', 'KATV')

    expected = {'resultaat': 'NOK', 'error': ANY}
    self.assertEqual(response, expected)
    assert mock_requestsget.called

  @patch('requests.get', side_effect=ConnectionError)
  def test_haalwaterstand_connectionerror(self, mock_requestsget):
    """ Test van een connection error bij het ophalen """
    response = waterstand.haalwaterstand('Katerveer', 'KATV')

    expected = {'resultaat': 'NOK', 'error': ANY}
    self.assertEqual(response, expected)
    assert mock_requestsget.called

  @patch('requests.get', side_effect=Timeout())
  def test_haalwaterstand_timeout(self, mock_requestsget):
    """ Test van een Time-out bij het ophalen """
    response = waterstand.haalwaterstand('Katerveer', 'KATV')

    expected = {'resultaat': 'NOK', 'error': ANY}
    self.assertEqual(response, expected)
    assert mock_requestsget.called

  @patch('requests.get', side_effect=RequestException())
  def test_haalwaterstand_requestexception(self, mock_requestsget):
    """ Test van een algemene exceptie bij het ophalen """
    response = waterstand.haalwaterstand('Katerveer', 'KATV')

    expected = {'resultaat': 'NOK', 'error': ANY}
    self.assertEqual(response, expected)
    assert mock_requestsget.called



"""
Ophalen van de waterstand. Gegevens komen van https://waterinfo.rws.nl/ en de gegevens voor vandaag
en morgen worden er uit gehaald.
"""
from datetime import datetime, timedelta

import requests


def leesjson(url: str) -> dict:
  """
  Haal JSON van de URL op
  :param url: URL waar de JSON opgehaald moet worden
  :type url: str
  :return: JSON
  :rtype: dict
  """
  try:
    headers = {'Accept': 'application/json'}
    req = requests.get(url, headers=headers, verify=False, timeout=10, allow_redirects=False)
    contentjson = req.json()
    contentjson['resultaat'] = 'OK'
    return contentjson
  except (ConnectionError,
          TimeoutError,
          requests.exceptions.HTTPError,
          requests.exceptions.RequestException) as error:
    return {'resultaat': 'NOK',
            'error': f'Error: Data ophalen mislukt vanwege {error}\nURL: {url}'}


def leeswaterstandjson(naam: str, afkorting: str) -> dict:
  """
  Lees de informatie van bepaalde locatie van de API van RWS.
  :param naam: Naam van de locatie
  :type naam: str
  :param afkorting: Afkorting van de locatie
  :type afkorting: str
  :return: JSON met de waardes van de waterstanden
  :rtype: dict
  """
  url = 'https://waterinfo.rws.nl/api/chart/get' + \
        f'?mapType=waterhoogte&locationCodes={naam}({afkorting})&values=-48,48'
  return leesjson(url)


def bepaalstanden(waterstandjson: dict) -> dict:
  """
  Haal de noodzakelijke gegevens van de waterstand uit de gegevens van RWS.
  :param waterstandjson: Dict met de gegevens van de waterstanden
  :type waterstandjson: dict
  :return: JSON met de waardes van de waterstanden
  :rtype: dict
  """
  if waterstandjson['resultaat'] == 'NOK':
    return waterstandjson
  laatstetijdgemeten = waterstandjson['t0']
  gemetenstanden = waterstandjson['series'][0]['data']
  voorspeldestanden = waterstandjson['series'][1]['data']

  hoogtenu = -999
  for stand in gemetenstanden:
    if stand['dateTime'] == laatstetijdgemeten:
      hoogtenu = stand['value']

  if laatstetijdgemeten.endswith('Z'):
    tijdpatroon = '%Y-%m-%dT%H:%M:%SZ'
    deltatijd = 1
  else:
    tijdpatroon = '%Y-%m-%dT%H:%M:%S+02:00'
    deltatijd = 1

  laatstetijdobj = datetime.strptime(laatstetijdgemeten, tijdpatroon) \
                   + timedelta(hours=deltatijd)
  weergavetijd = laatstetijdobj.strftime('%d-%m %H:%M')
  morgenobj = laatstetijdobj + timedelta(days=1)
  morgentekst = morgenobj.strftime(tijdpatroon)

  hoogtemorgen = hoogtenu
  for stand in voorspeldestanden:
    if stand['dateTime'] == morgentekst:
      hoogtemorgen = stand['value']
  returnjson = {'resultaat': 'OK',
                'tijd': weergavetijd,
                'nu': hoogtenu,
                'morgen': hoogtemorgen}
  return returnjson


def haalwaterstand(naam: str, afkorting: str) -> dict:
  """
  Haal de waterstand van een locatie bij RWS en haal de noodzakelijke waarden daar uit.
  :param naam: Naam van de locatie
  :type naam: str
  :param afkorting: Afkorting van de locatie
  :type afkorting: str
  :return: JSON met de waardes van de waterstanden
  :rtype: dict
  """
  contentjson = leeswaterstandjson(naam, afkorting)
  return bepaalstanden(contentjson)
